/*
  ConExec v1.11 - Shell Executer for ConTEXT editor by Eden Kirin
  (C) Copyright 2001-2002
  Written by Arno van Amersfoort

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  Target compiler : Borland C++ 5.5 for Win32 or MinGW32 (msvcrt version)
  Last modified   : 20-03-2002
*/
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>

char *doscommand = NULL;
int pause = 0;
DWORD priority = NORMAL_PRIORITY_CLASS;

char *amstrcat(const char *str, ...)
{
  static char *s = NULL;
  int size = 0;
  char *arg;
  va_list ap;

  if (s!=NULL) free(s);

  va_start(ap, str);
  while ((arg=va_arg(ap, char *)) != NULL) size+=strlen(arg);
  va_end(ap);

  if ((s = (char *) malloc(size+strlen(str)+1)) == NULL)
  {
    printf("Out of memory\n");
    printf("Program abnormally terminated\n");
    exit(1);
  }

  strcpy(s, str);

  va_start(ap, str);
  while ((arg=va_arg(ap, char *)) != NULL) strcat(s, arg);
  va_end(ap);

  return s;
}


char *astrcpy(char **newstr, const char *string)
{
  if (*newstr!=NULL) free(*newstr);

  if (string==NULL)
  {
    if ((*newstr = (char *) malloc(1)) == NULL)
    {
      printf("Out of memory\n");
      printf("Program abnormally terminated\n");
      exit(1);
    }
    strcpy(*newstr, "");
  }
  else
  {
    if ((*newstr = (char *) malloc(strlen(string)+1)) == NULL)
    {
      printf("Out of memory\n");
      printf("Program abnormally terminated\n");
      exit(1);
    }
    strcpy(*newstr, string);
  }

  return *newstr;
}


char *mid(const char *string, const unsigned int start, const unsigned int len)
{
  static char *s = NULL;

  if (s!=NULL) free(s);
  if ((s = (char *) malloc(len+1)) == NULL)
  {
    printf("Out of memory\n");
    printf("Program abnormally terminated\n");
    exit(1);
  }

  if (start>strlen(string))
  {
    s[0]='\0';
  }
  else
  {
    strncpy (s,string+start,len);
    s[len]='\0';
  }
  return s;
}


void ProcessCommandLine(char *in)
{
  unsigned int x;
  int b=0;
  int last=0;
  int argc=0;
  char *argument;

  astrcpy(&doscommand, "");

  for (x=0; x<=strlen(in); x++)
  {
    if (in[x]=='\"') b=!b;
    else
    if ((x==strlen(in)) || (in[x]==' ' && !b))
    {
      argc++;
      argument=mid(in, last, x-last);

      if (argc!=1)
      {
        if ((stricmp(argument, "-p")==0 || stricmp(argument, "/p")==0 || stricmp(argument, "--p")==0) && (strcmp(doscommand,"")==0))
        {
          pause=1;
        }
        else
        if ((stricmp(argument, "-i")==0 || stricmp(argument, "/i")==0 || stricmp(argument, "--i")==0) && (strcmp(doscommand,"")==0))
        {
          priority=IDLE_PRIORITY_CLASS;
        }
        else
        {
          if (x<=strlen(in)) astrcpy(&doscommand, in+last);     /* Get DOS command */
          return;
        }
      }
      last=x+1;
    }
  }
}


DWORD Spawn(char *CmdLine)
{
  STARTUPINFO si;
  PROCESS_INFORMATION pi;
  DWORD ExitCode;
  char *ExecCommand;

  if (GetVersion()<0x80000000 && LOBYTE(LOWORD(GetVersion()))<=4) /* Windows NT 3 or 4 environment ?? */
  {
    /* Surround the whole shell command with "" else NT's CMD.EXE will not work! */
    ExecCommand=amstrcat("cmd.exe /c \"", CmdLine, "\"", NULL);
  }
  else
  {
    /* Do a normal Windows 9X command without "" */
    ExecCommand=CmdLine;
  }

  memset(&si, 0, sizeof(si));
  si.cb = sizeof(si);

  if (CreateProcess(NULL, ExecCommand, NULL, NULL, TRUE, priority, NULL, NULL, &si, &pi)!=0)
  {
    WaitForSingleObject(pi.hProcess, INFINITE);
    GetExitCodeProcess(pi.hProcess, &ExitCode);
    CloseHandle(pi.hThread);
    CloseHandle(pi.hProcess);
    return 0; /* used to be: return ExitCode; */
  }
  else
  return 1;
}


int main(void)
{
  ProcessCommandLine(GetCommandLine()); /* Process the command line */

  if (strlen(doscommand)==0)
  {
    printf("ConExec v1.11 - ConTEXT Shell Executer - *FREEWARE* - NO WARRANTY!\n");
    printf("(C)Copyright 2001-2002 - Written by Arno van Amersfoort\n\n");
    printf("Usage: conexec.exe [-p] [-i] command [parameter(s)]\n");
    printf("-p           : Pause after execution\n");
    printf("-i           : Run command with idle priority (default is normal priority)\n");
    printf("command      : Command to execute via command interpreter\n");
    printf("parameter(s) : Parameter(s) to pass to the command\n");
    printf("NOTE: Parameters or commands with spaces should be surrounded by quotes (\"\").\n");
    return 0;
  }

  if (Spawn(doscommand)!=0)
    MessageBox(0, amstrcat("Error executing: ", doscommand, ".", NULL), "Error", MB_OK+MB_ICONERROR);

  if (pause)
  {
    printf("\nFinished. Press any key to exit...\n");
    while (!kbhit()) Sleep(100);
  }

  return 0;
}

/*
   1.11 changes:
   -------------
   - Support for Idle-priority (-i) provided by Patrick Houweling. Thanks for the addition!
   - Changed sleep(100) to Sleep(100) to be compatible with BCC

   1.10a changes:
   --------------
   - fixed cpu eating while pausing (-p)

   1.10 (=1.10RC4) changes:
   ------------------------
   - fixed broken NT4 execution

   1.10RC3 changes:
   ----------------
   - changed spawn routine again

   1.10RC2 changes:
   ----------------
   - fixed wrong Windows Ver detect under W9X

   1.10RC1 changes:
   ----------------
   - removed system() error show in messagebox
   - changed the Spawn routine to be more compatible
   - some minor tweaks

   1.10BETAx changes:
   ------------------
   - changed code to be compilable with BC 5.5++
   - changed system() to use Spawn (CreateProcess) routine (this seems to fix the NAV/W9X issue)

   1.09 changes:
   -------------
   - cleaned up code
   - removed seperate args => now part of doscommand variable
   - bug fix: no parameters would cause an access violation
   - bug fix: command only without parameters didn't work
   - improved astrcpy NULL pointer handling

   1.08 changes:
   -------------
   - bug fix: buffer overflow for a command lines longer then 255 chars. Changed allocation to be dynamic
   - cleaned up processcommandline routine

   1.07 changes:
   -------------
   - cleaned up code
   - detect system() errors

   1.06 changes:
   -------------
   - Removed error window because the command interpreter only returns the child's returncode
     & not its own execute error.

   1.05 changes:
   -------------
   - Changed back to old system call: using non-existant batches/binaries under W9X
     would freeze Context (So would the original conexec.exe).
     Using system() instead of CreateProcess() is (a bit) slower but far more reliable.
   - Renamed exec to ConExec

   1.04 changes:
   -------------
   - Changed execute function, now uses createprocess API (Ported from Eden's conexec)
   - Bug fix: interpret problem of commandline for W9X
   - Now error is reported in Windows box
   - Now CMD.EXE is used if Windows NT is detected else a 'normal' execute is done

   1.03 changes:
   -------------
   - Bug fix: Didn't work under Windows 9X. Now autodetects what kind of command
     interpreter is used.
   - Minor bug fix: Usage screen didn't work the way it should.
   - Minor bug fix: Fixed spaces problem with no parameters.
*/

