program ReplaceNotepad;

uses
  Windows, SysUtils;

//------------------------------------------------------------------------------
// Version Information
//------------------------------------------------------------------------------

type
  TWindowsVersion =
   (wvUnknown, wvWin95, wvWin95OSR2, wvWin98, wvWin98SE, wvWinME,
    wvWinNT31, wvWinNT35, wvWinNT351, wvWinNT4, wvWin2000, wvWinXP, wvWin2003);
  TNtProductType = (ptUnknown, ptWorkStation, ptServer, ptAdvancedServer);

type
  TErrCode = (errNone, errDelete, errMove, errCopy, errUnknownWinVersion);

var
  WindowsPath  :array[0..MAX_PATH-1] of char;
  pWindowsPath :PChar;
  SystemPath   :array[0..MAX_PATH-1] of char;
  pSystemPath  :PChar;
  WinVer       :TWindowsVersion;
  ErrorCode    :TErrCode;
  ErrorMessage :string;

const
  NOTEPAD_FNAME = 'Notepad.exe';
  BAK_EXT       = '.bak';

  CRLF        = #13#10;
  DLG_CAPTION = 'Replace Windows Notepad';
  DLG_TEXT    = 'Setup will replace original Notepad.exe with fake which will call ConTEXT in case some '+
                'application has hardcoded call to Notepad.exe instead of application associated with '+
                'file extension. Original Notepad will be renamed to Notepad.exe.bak. You don''t have '+
                'to replace it now, you can find ReplaceNotepad.exe in ConTEXT install directory and '+
                'run it later. For more information about this replacement, read ReadMe.txt.'+CRLF+CRLF+
                'Do you want to continue?';

  ERR_CAPTION = 'Error';


function GetWindowsVersion: TWindowsVersion;
var
  TrimmedWin32CSDVersion: string;
begin
  Result := wvUnknown;
  TrimmedWin32CSDVersion := Trim(Win32CSDVersion);
  case Win32Platform of
    VER_PLATFORM_WIN32_WINDOWS:
      case Win32MinorVersion of
        0..9:
          if (TrimmedWin32CSDVersion = 'B') or (TrimmedWin32CSDVersion = 'C') then
            Result := wvWin95OSR2
          else
            Result := wvWin95;
        10..89:
          if TrimmedWin32CSDVersion = 'A' then
            Result := wvWin98SE
          else
            Result := wvWin98;
        90:
          Result := wvWinME;
      end;
    VER_PLATFORM_WIN32_NT:
      case Win32MajorVersion of
        3:
          case Win32MinorVersion of
            1:
              Result := wvWinNT31;
            5:
              Result := wvWinNT35;
            51:
              Result := wvWinNT351;
          end;
        4:
          Result := wvWinNT4;
        5:
          case Win32MinorVersion of
            0:
              Result := wvWin2000;
            1:
              Result := wvWinXP;
            2:
              Result := wvWin2003;
          end;
      end;
  end;
end;

procedure AddBackslash(p:PChar);
var
  n :integer;
begin
  n:=StrLen(p);

  if ((p+n)<>'\') then begin
    (p+n)^:='\';
    (p+(n+1))^:=#0;
  end;
end;

function CopyAndBackup(DestDir:string):boolean;
begin
  result:=TRUE;

  if not FileExists(DestDir+NOTEPAD_FNAME+'.bak') then begin
    result:=not FileExists(DestDir+NOTEPAD_FNAME) or MoveFile(PChar(DestDir+NOTEPAD_FNAME), PChar(DestDir+NOTEPAD_FNAME+'.bak'));
    if not result then begin
      ErrorCode:=errMove;
      ErrorMessage:='Error backing up "'+DestDir+NOTEPAD_FNAME+'" to "'+DestDir+NOTEPAD_FNAME+'.bak".';
    end;
  end;

  if result then begin
    result:=CopyFile(NOTEPAD_FNAME,PChar(DestDir+NOTEPAD_FNAME),FALSE);
    if not result then begin
      ErrorCode:=errCopy;
      ErrorMessage:='Error copying "'+NOTEPAD_FNAME+'" to "'+DestDir+NOTEPAD_FNAME+'".';
    end;
  end;
end;

function DeleteFile(FileName:string):boolean;
begin
  result:=not FileExists(FileName) or Windows.DeleteFile(PChar(FileName));

  if not result then begin
    ErrorCode:=errDelete;
    ErrorMessage:='Error deleting "'+FileName+'".';
  end;
end;

function Confirm:boolean;
begin
  result:=MessageBox(0, DLG_TEXT, DLG_CAPTION, MB_ICONQUESTION+MB_OKCANCEL)=IDOK;
end;

begin
  ErrorCode:=errNone;
  WinVer:=GetWindowsVersion;

  if not Confirm then EXIT;

  pWindowsPath:=@WindowsPath;
  pSystemPath:=@SystemPath;

  GetWindowsDirectory(pWindowsPath,SizeOf(WindowsPath));
  GetSystemDirectory(pSystemPath,SizeOf(SystemPath));

  AddBackslash(pWindowsPath);
  AddBackslash(pSystemPath);

  case WinVer of
    wvWin2000:
      begin
        if DeleteFile(PChar(pSystemPath+'dllcache\'+NOTEPAD_FNAME)) then begin
          CopyAndBackup(pWindowsPath);
          CopyAndBackup(pSystemPath);
        end;
      end;
    wvUnknown:
      begin
        ErrorCode:=errUnknownWinVersion;
      end;
    else begin
      CopyAndBackup(pWindowsPath);
      CopyAndBackup(pSystemPath);
    end;
  end;

  if (ErrorCode<>errNone) then
    MessageBox(0, PChar(ErrorMessage), ERR_CAPTION, MB_ICONERROR+MB_OK);
end.

