unit FileBrowser;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ExtCtrls, ComCtrls, Buttons, StdCtrls, DirLabel, DirView, IEComboBox,
  Registry, IniFiles, Common, multi_language, ShellAPI;

type
  THScrollListBox = class(TListBox)
  private
    MaxExtent: integer;     { maximum length of the strings in the listbox (in pixel) }
    function GetStringExtent( s: string ): integer;
  protected
    procedure OnCustomDrawItem(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
  public
    constructor Create( AOwner: TComponent ); override;
    function  AddString(s:string; Data:TObject):integer;
    procedure DeleteString(i:integer);
    procedure ClearList;
    procedure AdjustWidth;
  published
  end;

  TFileData = record
		FileName    :string[255];
                IsDirectory :boolean;
		Icon	    :TBitmap;
	      end;

  pTFileData = ^TFileData;

  TFileBrowsePanel = class(TCustomPanel)
  private
    pnTop	      :TPanel;
    pnDir	      :TPanel;
    labDir	      :TDirLabel;
    btnClose	      :TSpeedButton;
    imgsToolbar       :TImageList;
    popView	      :TPopUpMenu;
    ToolBar	      :TToolBar;
    btnUp	      :TToolButton;
    btnView	      :TToolButton;
    cbDisk	      :TIEDriveComboBox;
    expl	      :TDirView;
    splitterRight     :TSplitter;

    splitterMiddle    :TSplitter;
    pnFav	      :TPanel;
    pnFavCaption      :TPanel;
    pnFavButtons      :TPanel;
    btnFavAdd	      :TSpeedButton;
    btnFavRemove      :TSpeedButton;
    btnFavOpen	      :TSpeedButton;
    btnFavTogglePath  :TSpeedButton;
    lbFav	      :THScrollListBox;

    FChanged	      :boolean;
    FShowIcons	      :boolean;

    procedure SetShowIcons(Value:boolean);

    procedure pnDirResize(Sender: TObject);
    procedure btnUpClick(Sender: TObject);
    procedure btnFavAddClick(Sender: TObject);
    procedure btnFavRemoveClick(Sender: TObject);
    procedure btnFavOpenClick(Sender: TObject);
    procedure btnFavTogglePathClick(Sender: TObject);
    procedure popViewPopup(Sender: TObject);
    procedure miViewStyleClick(Sender: TObject);
    procedure splitterCanResize(Sender: TObject; var NewSize: Integer; var Accept: Boolean);
    procedure btnCloseClick(Sender: TObject);
    procedure explExecFile(Sender: TObject; Item: TListItem; var AllowExec: Boolean);
    procedure cbDiskChange(Sender: TObject);
    procedure explLoaded(Sender: TObject);
    procedure lbFavOnKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);

    procedure LoadSettings;
    procedure SaveSettings;
    procedure AddFileToFavorites(fname:string);
    function  RemoveFileFromFavorites(n:integer):boolean;
    procedure SortFavoriteItems;
    procedure LoadFavorites;
    procedure SaveFavorites;
    function  SetPath(Path:string; const TryDefault:boolean = TRUE):boolean;
    procedure SelectFavoritesItem(n:integer);
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property ShowIcons :boolean read FShowIcons write SetShowIcons;
  end;

implementation

uses
  editor, main;

const
  BOLD_BEG = #09;
  BOLD_END = #10;

  FAVORITES_SECTION = 'ConTEXT Favorites';
  FAVORITES_FNAME   = 'Favorites.ini';

type
  TFNameArray = array[0..127] of char;


////////////////////////////////////////////////////////////////////////////////////////////
//				       THScrollListBox
////////////////////////////////////////////////////////////////////////////////////////////

constructor THScrollListBox.Create( AOwner: TComponent );
begin
  inherited Create( AOwner );
  OnDrawItem:=OnCustomDrawItem;
  MaxExtent := 0;
end;


function THScrollListBox.GetStringExtent( s: string ): integer;
var
  dwExtent:	      DWORD;
  hDCListBox:	      HDC;
  hFontOld, hFontNew: HFONT;
  tm:		      TTextMetric;
  Size: 	      TSize;
begin
  hDCListBox := GetDC( Handle );
  hFontNew := SendMessage( Handle, WM_GETFONT, 0, 0 );
  hFontOld := SelectObject( hDCListBox, hFontNew );
  GetTextMetrics( hDCListBox, tm );

  { the following two lines should be modified for Delphi 1.0: call GetTextExtent }
  GetTextExtentPoint32( hDCListBox, PChar(s), Length(s), Size );
  dwExtent := Size.cx + tm.tmAveCharWidth;

  SelectObject( hDCListBox, hFontOld );
  ReleaseDC( Handle, hDCListBox );

  GetStringExtent := LOWORD( dwExtent );
end;

procedure THScrollListBox.OnCustomDrawItem(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  bmp	    :TBitmap;
  ofs	    :Integer;
  rec	    :pTFileData;
begin
  bmp:=nil;

  with Canvas do begin
    FillRect(Rect);
    ofs:=2;
    rec:=pTFileData(Items.Objects[Index]);

    if Assigned(rec^.Icon) then
      bmp:=rec^.Icon;

    if (bmp<>nil) then begin
      BrushCopy(Bounds(Rect.Left + 2, Rect.Top+1, bmp.Width, bmp.Height),
		bmp, Bounds(0, 0, bmp.Width, bmp.Height), BMP_MASK);
      ofs:=bmp.width+6;
    end;
    TextOut(Rect.Left+ofs,Rect.Top+2, Items[Index])
  end;
end;

function THScrollListBox.AddString(s:string; Data:TObject):integer;
var
  StrExtent: integer;
begin
  StrExtent := GetStringExtent( s );
  if StrExtent > MaxExtent then
  begin
    MaxExtent := StrExtent;
    SendMessage( Handle, LB_SETHORIZONTALEXTENT, MaxExtent, 0 );
  end;

  { adds the string to the listbox }
  result:=Items.AddObject(s,Data);
end;

procedure THScrollListBox.DeleteString(i:integer);
begin
  Items.Delete(i);
  AdjustWidth;
end;

procedure THScrollListBox.ClearList;
begin
  MaxExtent := 0;
	SendMessage( Handle, LB_SETHORIZONTALEXTENT, 0, 0 );

	{ scrolls the listbox to the left }
	SendMessage( Handle, WM_HSCROLL, SB_TOP, 0 );

  { clears the listbox }
  Items.Clear;
  AdjustWidth;
end;

procedure THScrollListBox.AdjustWidth;
var
  i		 :integer;
  StrExtent	 :integer;
begin
  SendMessage( Handle, WM_HSCROLL, SB_TOP, 0 );

  MaxExtent:=0;
  i:=0;
  while (i<Items.Count) do begin
    StrExtent:=GetStringExtent(Items[i]);
    if (StrExtent>MaxExtent) then
      MaxExtent := StrExtent;
    inc(i);
  end;

  SendMessage( Handle, LB_SETHORIZONTALEXTENT, MaxExtent, 0 );
end;


////////////////////////////////////////////////////////////////////////////////////////////
//				Constructor, destructor
////////////////////////////////////////////////////////////////////////////////////////////
//------------------------------------------------------------------------------------------
constructor TFileBrowsePanel.Create(AOwner: TComponent);
var
  menuitem :TMenuItem;
  bmp	   :TBitmap;
begin
  inherited;
  // self properties
  Align:=alLeft;
  BevelInner:=bvNone;
  BevelOuter:=bvRaised;
  Width:=200;
  FullRepaint:=FALSE;
  Visible:=FALSE;
  Parent:=TWinControl(AOwner);


  // View style popup menu /////////////////////

  popView:=TPopupMenu.Create(SELF);
  popView.OnPopup:=popViewPopup;

  menuitem:=TMenuItem.Create(SELF);
  with menuitem do begin
    Caption:=mlStr(ML_EXPL_LARGE_ICONS,'Lar&ge Icons');
    Tag:=0;
    GroupIndex:=1;
    RadioItem:=TRUE;
    OnClick:=miViewStyleClick;
  end;
  popView.Items.Add(menuitem);

  menuitem:=TMenuItem.Create(SELF);
  with menuitem do begin
    Caption:=mlStr(ML_EXPL_SMALL_ICONS,'S&mall Icons');
    Tag:=1;
    GroupIndex:=1;
    RadioItem:=TRUE;
    OnClick:=miViewStyleClick;
  end;
  popView.Items.Add(menuitem);

  menuitem:=TMenuItem.Create(SELF);
  with menuitem do begin
    Caption:=mlStr(ML_EXPL_LIST,'&List');
    Tag:=2;
    GroupIndex:=1;
    RadioItem:=TRUE;
    OnClick:=miViewStyleClick;
  end;
  popView.Items.Add(menuitem);

  menuitem:=TMenuItem.Create(SELF);
  with menuitem do begin
    Caption:=mlStr(ML_EXPL_DETAILS,'&Details');
    Tag:=3;
    GroupIndex:=1;
    RadioItem:=TRUE;
    OnClick:=miViewStyleClick;
  end;
  popView.Items.Add(menuitem);


  // Favorites /////////////////////////////////

  pnFav:=TPanel.Create(SELF);
  with pnFav do begin
    Align:=alBottom;
    BevelInner:=bvNone;
    BevelOuter:=bvNone;
    FullRepaint:=FALSE;
    Height:=190;
    Parent:=SELF;
  end;

  pnFavCaption:=TPanel.Create(pnFav);
  with pnFavCaption do begin
    Align:=alTop;
    Alignment:=taLeftJustify;
    Caption:=' '+mlStr(ML_EXPL_CAPT_FAVORITES,'Favorites');
    BevelInner:=bvNone;
    BevelOuter:=bvNone;
    FullRepaint:=FALSE;
    Height:=15;
    Parent:=pnFav;
  end;

  pnFavButtons:=TPanel.Create(pnFavCaption);
  with pnFavButtons do begin
    Align:=alRight;
    BevelInner:=bvNone;
    BevelOuter:=bvNone;
    FullRepaint:=FALSE;
    Width:=67;
    Parent:=pnFavCaption;
  end;

  btnFavAdd:=TSpeedButton.Create(pnFavButtons);
  with btnFavAdd do begin
    Flat:=TRUE;
    Left:=1;
    Height:=14;
    Width:=14;
    Hint:=mlStr(ML_EXPL_HINT_ADD,'Add selected files from File Explorer to favorites');
    ShowHint:=TRUE;
    Glyph.LoadFromResourceName(hInstance,'BMP_FAVADD');
    OnClick:=btnFavAddClick;
    Parent:=pnFavButtons;
  end;

  btnFavRemove:=TSpeedButton.Create(pnFavButtons);
  with btnFavRemove do begin
    Flat:=TRUE;
    Left:=17;
    Height:=14;
    Width:=14;
    Hint:=mlStr(ML_EXPL_HINT_REMOVE,'Remove selected files from favorites');
    ShowHint:=TRUE;
    Glyph.LoadFromResourceName(hInstance,'BMP_FAVREMOVE');
    OnClick:=btnFavRemoveClick;
    Parent:=pnFavButtons;
  end;

  btnFavOpen:=TSpeedButton.Create(pnFavButtons);
  with btnFavOpen do begin
    Flat:=TRUE;
    Left:=33;
    Height:=14;
    Width:=14;
    Hint:=mlStr(ML_EXPL_HINT_OPEN,'Open selected files');
    ShowHint:=TRUE;
    Glyph.LoadFromResourceName(hInstance,'BMP_FAVOPEN');
    OnClick:=btnFavOpenClick;
    Parent:=pnFavButtons;
  end;

  btnFavTogglePath:=TSpeedButton.Create(pnFavButtons);
  with btnFavTogglePath do begin
    Flat:=TRUE;
    Left:=49;
    Height:=14;
    Width:=14;
    AllowAllUp:=TRUE;
    GroupIndex:=1;
    Hint:=mlStr(ML_EXPL_HINT_TOGGLEPATH,'Toggle show/hide path');
    ShowHint:=TRUE;
    Glyph.LoadFromResourceName(hInstance,'BMP_FAVTOGGLEPATH');
    OnClick:=btnFavTogglePathClick;
    Parent:=pnFavButtons;
  end;

  lbFav:=THScrollListBox.Create(pnFav);
  with lbFav do begin
    Align:=alClient;
    MultiSelect:=TRUE;
    ShowIcons:=TRUE;
    OnDblClick:=btnFavOpenClick;
    OnKeyDown:=lbFavOnKeyDown;
    Parent:=pnFav;
  end;


  // Toolbar ///////////////////////////////////

  imgsToolbar:=TImageList.Create(SELF);
  with imgsToolbar do begin
    bmp:=TBitmap.Create;
    bmp.LoadFromResourceName(hInstance,'BMP_FAVLEVELUP');
    AddMasked(bmp,clAqua);
    bmp:=TBitmap.Create;
    bmp.LoadFromResourceName(hInstance,'BMP_FAVBROWSESTYLE');
    AddMasked(bmp,clAqua);
  end;

  Toolbar:=TToolbar.Create(SELF);
  with Toolbar do begin
    Align:=alTop;
    Flat:=TRUE;
    Parent:=SELF;
    Images:=imgsToolbar;
    Wrapable:=FALSE;
  end;

  btnView:=TToolButton.Create(Toolbar);
  with btnView do begin
    Style:=tbsDropDown;
    DropDownMenu:=popView;
    Hint:=mlStr(ML_EXPL_VIEWSTYLE,'View style');
    ShowHint:=TRUE;
    ImageIndex:=1;
    Height:=22;
    Width:=36;
    Parent:=Toolbar;
  end;

  btnUp:=TToolButton.Create(Toolbar);
  with btnUp do begin
    Style:=tbsButton;
    Hint:=mlStr(ML_EXPL_UPONELEVEL,'Up one level');
    ShowHint:=TRUE;
    ImageIndex:=0;
    Height:=22;
    Width:=24;
    OnClick:=btnUpClick;
    Parent:=Toolbar;
  end;

  cbDisk:=TIEDriveComboBox.Create(Toolbar);
  with cbDisk do begin
    Align:=alTop;
    Width:=125;
    OnChange:=cbDiskChange;
    Parent:=Toolbar;
  end;


  // Directory panel ///////////////////////////

  pnDir:=TPanel.Create(SELF);
  with pnDir do begin
    Align:=alTop;
    BevelInner:=bvNone;
    BevelOuter:=bvNone;
    FullRepaint:=FALSE;
    Height:=17;
    OnResize:=pnDirResize;
    Parent:=SELF;
  end;

  labDir:=TDirLabel.Create(pnDir);
  with labDir do begin
    Alignment:=taLeftJustify;
    Left:=1;
    Top:=2;
    Width:=165;
    Parent:=pnDir;
  end;

  btnClose:=TSpeedButton.Create(pnDir);
  with btnClose do begin
    SetBounds(1,1,12,12);
    Flat:=TRUE;
    Glyph:=fmMain.imgCloseBtn.Picture.Bitmap;
    OnClick:=btnCloseClick;
    Parent:=pnDir;
  end;


  // Tree //////////////////////////////////////
  expl:=TDirView.Create(SELF);
  with expl do begin
    Align:=alClient;
    DimmHiddenFiles:=TRUE;
    IconOptions.Arrangement:=iaTop;
    IconOptions.AutoArrange:=TRUE;
    MultiSelect:=TRUE;
    ReadOnly:=TRUE;
    RowSelect:=TRUE;
    UseIconCache:=TRUE;
    UseIconUpdateThread:=TRUE;
    ViewStyle:=vsIcon;
    WatchForChanges:=TRUE;
    HideSelection:=FALSE;
    OnExecFile:=explExecFile;
    OnLoaded:=explLoaded;

    Parent:=SELF;
  end;

  splitterMiddle:=TSplitter.Create(SELF);
  with splitterMiddle do begin
    ResizeStyle:=rsUpdate;
    Height:=3;
    MinSize:=50;
    Beveled:=TRUE;
    Align:=alBottom;
    Visible:=TRUE;
    OnCanResize:=splitterCanResize;
    Parent:=SELF;
  end;

  splitterRight:=TSplitter.Create(AOwner);
  with splitterRight do begin
    ResizeStyle:=rsUpdate;
    MinSize:=50;
    Beveled:=TRUE;
    Align:=alLeft;
    OnCanResize:=splitterCanResize;
    Visible:=TRUE;
  end;

  LoadSettings;

  cbDisk.TabOrder:=2;
  expl.TabOrder:=1;
  lbFav.TabOrder:=0;

  // napokon se pokai
  splitterRight.Parent:=TWinControl(AOwner);
  splitterRight.Left:=Width+2;
  Visible:=TRUE;
end;
//------------------------------------------------------------------------------------------
destructor TFileBrowsePanel.Destroy;
begin
  SaveSettings;

  // prvo njega ubijemo da se ne bi aktivirao njegov notification o promjeni fajlova dok snimamo
  // favoritese usred njegovog ubijanja - fino radi, ne diraj
  expl.Free;

  SaveFavorites;
  while RemoveFileFromFavorites(0) do ;

  lbFav.Free;
  btnFavAdd.Free;
  btnFavRemove.Free;
  btnFavOpen.Free;
  btnFavTogglePath.Free;
  labDir.Free;
  btnClose.Free;

//  ubijanje kontrola na toolbaru dize exception (?)
//  btnUp.Free;
//  btnView.Free;
//  cbDisk.Free;
//  imgsToolbar.Free;

  popView.Free;
  splitterRight.Free;
  splitterMiddle.Free;
  pnFavButtons.Free;
  ToolBar.Free;
  pnFavCaption.Free;
  pnFav.Free;
  pnDir.Free;
  pnTop.Free;

  inherited;
end;
//------------------------------------------------------------------------------------------


////////////////////////////////////////////////////////////////////////////////////////////
//				 Property functions
////////////////////////////////////////////////////////////////////////////////////////////
//------------------------------------------------------------------------------------------
procedure TFileBrowsePanel.SetShowIcons(Value:boolean);
begin
  if (FShowIcons<>Value) then begin
    FShowIcons:=Value;

    if Value then begin
      lbFav.Style:=lbOwnerDrawVariable;

      if (fmMain.DefaultlbItemHeight>18) then
	lbFav.ItemHeight:=fmMain.DefaultlbItemHeight
      else
	lbFav.ItemHeight:=18;
    end else begin
      lbFav.Style:=lbStandard;
      lbFav.ItemHeight:=fmMain.DefaultlbItemHeight;
    end;
  end;
end;
//------------------------------------------------------------------------------------------


////////////////////////////////////////////////////////////////////////////////////////////
//				     Functions
////////////////////////////////////////////////////////////////////////////////////////////
//------------------------------------------------------------------------------------------
procedure TFileBrowsePanel.LoadSettings;
var
  reg	  :TRegistry;
begin
  reg:=TRegistry.Create;

  try
    with reg do begin
      OpenKey(REG_KEY+'FileExplorer',TRUE);

      Width:=ReadRegistryInteger(reg,'Width',200);

      with expl do begin
	SetPath(ReadRegistryString(reg,'Dir',StartDir));

	ViewStyle:=TViewStyle(ReadRegistryInteger(reg,'ViewStyle',0));
	ColWidthAttr:=ReadRegistryInteger(reg,'ColWidthAttr',45);
	ColWidthDate:=ReadRegistryInteger(reg,'ColWidthDate',130);
	ColWidthName:=ReadRegistryInteger(reg,'ColWidthName',150);
	ColWidthSize:=ReadRegistryInteger(reg,'ColWidthSize',80);
	ColWidthType:=ReadRegistryInteger(reg,'ColWidthType',125);
	btnFavTogglePath.Down:=ReadRegistryBool(reg,'FavShowPath',TRUE);
      end;
    end;
  finally
    reg.Free;
  end;

  expl.ReLoad(TRUE);
  LoadFavorites;
end;
//------------------------------------------------------------------------------------------
procedure TFileBrowsePanel.SaveSettings;
var
  reg :TRegistry;
begin
  reg:=TRegistry.Create;

  try
    with reg do begin
      OpenKey(REG_KEY+'FileExplorer',TRUE);

      WriteInteger('Width',Width);

      with expl do begin
	WriteString('Dir',Path);
	WriteInteger('ColWidthAttr',ColWidthAttr);
	WriteInteger('ColWidthDate',ColWidthDate);
	WriteInteger('ColWidthName',ColWidthName);
	WriteInteger('ColWidthSize',ColWidthSize);
	WriteInteger('ColWidthType',ColWidthType);
	WriteBool('FavShowPath',btnFavTogglePath.Down);
      end;

      WriteInteger('ViewStyle',ord(expl.ViewStyle));
    end;
  finally
    reg.Free;
  end;
end;
//------------------------------------------------------------------------------------------
procedure TFileBrowsePanel.AddFileToFavorites(fname:string);
var
  s	   :string;
  i	   :integer;
  rec	   :pTFileData;
  attr     :integer;
begin
  if (Length(fname)=0) then EXIT;

  i:=0; s:=UpperCase(fname);
  while (i<lbFav.Items.Count) do begin
    if (s=UpperCase(pTFileData(lbFav.Items.Objects[i])^.FileName)) then
      EXIT;
    inc(i);
  end;

  attr:=FileGetAttr(fname);

  if (attr<>-1) then begin
    if btnFavTogglePath.Down then
      s:=fname
    else
      s:=ExtractFileName(fname);

    rec:=AllocMem(SizeOf(TFileData));
    rec^.FileName:=fname;
    rec^.Icon:=Get16x16FileIconBmp(fname);
    rec^.IsDirectory:=(attr and faDirectory)>0;

    lbFav.AddString(s,pointer(rec));
  end;
end;
//------------------------------------------------------------------------------------------
function TFileBrowsePanel.RemoveFileFromFavorites(n:integer):boolean;
begin
  result:=n<lbFav.Items.Count;
  if not result then EXIT;

  if Assigned(lbFav.Items.Objects[n]) then begin

    if Assigned(pTFileData(lbFav.Items.Objects[n])^.Icon) then
      pTFileData(lbFav.Items.Objects[n])^.Icon.Free;

    FreeMem(pointer(lbFav.Items.Objects[n]),SizeOf(TFileData));
  end;

  lbFav.DeleteString(n);
end;
//------------------------------------------------------------------------------------------
procedure TFileBrowsePanel.SortFavoriteItems;
var
  i            :integer;
  n            :integer;
  strDirs      :TStringList;
  strFiles     :TStringList;
  SelectedItem :pointer;
begin
  if (lbFav.Items.Count>1) then begin
    n:=lbFav.ItemIndex;

    if (n>-1) then
      SelectedItem:=pTFileData(lbFav.Items.Objects[n])
    else
      SelectedItem:=nil;

    strDirs:=TStringList.Create;
    strFiles:=TStringList.Create;

    for i:=0 to lbFav.Items.Count-1 do
      if (pTFileData(lbFav.Items.Objects[i])^.IsDirectory) then
        strDirs.AddObject(lbFav.Items[i],lbFav.Items.Objects[i])
      else
        strFiles.AddObject(lbFav.Items[i],lbFav.Items.Objects[i]);

    lbFav.Items.BeginUpdate;
    lbFav.Items.Clear;

    for i:=0 to strDirs.Count-1 do
      lbFav.AddString(strDirs[i],strDirs.Objects[i]);
    for i:=0 to strFiles.Count-1 do
      lbFav.AddString(strFiles[i],strFiles.Objects[i]);

    lbFav.Items.EndUpdate;

    SelectFavoritesItem(lbFav.Items.IndexOfObject(SelectedItem));

    strDirs.Free;
    strFiles.Free;
  end;
end;
//------------------------------------------------------------------------------------------
procedure TFileBrowsePanel.LoadFavorites;
var
  iniF		:TIniFile;
  i		:integer;
  s		:string;
begin
  lbFav.Items.BeginUpdate;
  lbFav.Items.Clear;

  iniF:=TIniFile.Create(StartDir+FAVORITES_FNAME);

  try
    with iniF do begin
      i:=0;
      s:='';
      repeat
	s:=ReadString(FAVORITES_SECTION,'File'+IntToStr(i),'');
	AddFileToFavorites(s);
	inc(i);
      until (Length(s)=0);
    end;
  finally
    iniF.Free;
    lbFav.Items.EndUpdate;
  end;

  btnFavTogglePathClick(SELF);

  SortFavoriteItems;
  FChanged:=FALSE;
end;
//------------------------------------------------------------------------------------------
procedure TFileBrowsePanel.SaveFavorites;
var
  iniF :TIniFile;
  i    :integer;
begin
  if not FChanged then EXIT;

  DeleteFile(StartDir+FAVORITES_FNAME);

  iniF:=TIniFile.Create(StartDir+FAVORITES_FNAME);

  try
    with iniF do begin
      for i:=0 to lbFav.Items.Count-1 do
	WriteString(FAVORITES_SECTION,'File'+IntToStr(i),pTFileData(lbFav.Items.Objects[i])^.FileName);
    end;
  finally
    iniF.Free;
    lbFav.Items.EndUpdate;
  end;

  FChanged:=FALSE;
end;
//------------------------------------------------------------------------------------------
function TFileBrowsePanel.SetPath(Path:string; const TryDefault:boolean = TRUE):boolean;
var
  attr  :integer;
begin
  attr:=FileGetAttr(Path);
  result:=(attr<>-1) and ((attr and faDirectory)>0);

  if result and (Length(path)>0) then begin
    Path[1]:=UpperCase(Path[1])[1];

    expl.Path:=Path;

    if (Length(expl.Path)>0) then
      cbDisk.Drive:=expl.Path[1];
  end else
    if TryDefault then
      SetPath('C:\',FALSE);
end;
//------------------------------------------------------------------------------------------
procedure TFileBrowsePanel.SelectFavoritesItem(n:integer);
var
  i :integer;
begin
  for i:=0 to lbFav.Items.Count-1 do
    lbFav.Selected[i]:=FALSE;

  if (n>lbFav.Items.Count-1) then
    n:=lbFav.Items.Count-1;

  lbFav.ItemIndex:=n;
  lbFav.Selected[n]:=TRUE;

  if (n=0) then begin
    SendMessage(lbFav.Handle,WM_LBUTTONDOWN,0,$00010001);
    SendMessage(lbFav.Handle,WM_LBUTTONUP,0,$00010001);
  end;
end;
//------------------------------------------------------------------------------------------


////////////////////////////////////////////////////////////////////////////////////////////
//					 Events
////////////////////////////////////////////////////////////////////////////////////////////
//------------------------------------------------------------------------------------------
procedure TFileBrowsePanel.pnDirResize(Sender: TObject);
begin
  if not Assigned(btnClose) then EXIT;

  btnClose.Left:=pnDir.Width-btnClose.Width-2;
  labDir.Width:=btnClose.Left-2;
end;
//------------------------------------------------------------------------------------------
procedure TFileBrowsePanel.btnUpClick(Sender: TObject);
var
  i	 :integer;
  deepth :integer;
  s	 :string;
begin
  deepth:=0;
  s:=expl.Path;
  for i:=1 to Length(s) do
    if (s[i]='\') then inc(deepth);

  if (deepth>0) then begin
    i:=Length(s)-1;
    while (i>0) and (s[i]<>'\') do
      dec(i);

    SetLength(s,i);
  end;

  expl.Path:=s;
end;
//------------------------------------------------------------------------------------------
procedure TFileBrowsePanel.btnFavAddClick(Sender: TObject);
var
  i   :integer;
  str :TStringList;
begin
  str:=TStringList.Create;

  lbFav.Items.BeginUpdate;

  try
    expl.BuildFileList(str);

    for i:=0 to str.Count-1 do begin
      AddFileToFavorites(str[i]);
      FChanged:=TRUE;
    end;
  finally
    lbFav.Items.EndUpdate;
    SortFavoriteItems;
    str.Free;
  end;
end;
//------------------------------------------------------------------------------------------
procedure TFileBrowsePanel.btnFavRemoveClick(Sender: TObject);
var
  i :integer;
  n :integer;
begin
  lbFav.Items.BeginUpdate;

  if (lbFav.SelCount>1) then
    n:=0
  else
    n:=lbFav.ItemIndex;

  try
    i:=0;
    while (i<lbFav.Items.Count) do begin
      if (lbFav.Selected[i]) then begin
	RemoveFileFromFavorites(i);
	FChanged:=TRUE;
      end else
	inc(i);
    end;

    SelectFavoritesItem(n);
  finally
    lbFav.Items.EndUpdate;
  end;
end;
//------------------------------------------------------------------------------------------
procedure TFileBrowsePanel.btnFavOpenClick(Sender: TObject);
var
  i             :integer;
  rec           :pTFileData;
  SelectedCount :integer;
  failed        :boolean;
begin
  SelectedCount:=0;
  for i:=0 to lbFav.Items.Count-1 do
    if lbFav.Selected[i] then
      inc(SelectedCount);

  i:=0;
  while (i<lbFav.Items.Count) do begin
    if (lbFav.Selected[i]) then begin
      rec:=pTFileData(lbFav.Items.Objects[i]);
      failed:=FALSE;

      if rec^.IsDirectory then begin
        if (SelectedCount=1) then
          failed:=not SetPath(rec^.FileName);
      end else begin
        failed:=not fmMain.OpenFile(rec^.FileName);
      end;

      if failed then begin
        RemoveFileFromFavorites(i);
        dec(i);
        FChanged:=TRUE;
      end;

    end;
    inc(i);
  end;
end;
//------------------------------------------------------------------------------------------
procedure TFileBrowsePanel.btnFavTogglePathClick(Sender: TObject);
var
  i :integer;
begin
  lbFav.Items.BeginUpdate;

  try
    for i:=0 to lbFav.Items.Count-1 do begin
      if Assigned(lbFav.Items.Objects[i]) then begin
	if btnFavTogglePath.Down then
	  lbFav.Items[i]:=pTFileData(lbFav.Items.Objects[i])^.FileName
	else
	  lbFav.Items[i]:=ExtractFileName(pTFileData(lbFav.Items.Objects[i])^.FileName);
      end;
    end;
  finally
    lbFav.Items.EndUpdate;
  end;

  lbFav.AdjustWidth;
end;
//------------------------------------------------------------------------------------------
procedure TFileBrowsePanel.popViewPopup(Sender: TObject);
begin
  if (ord(expl.ViewStyle) in [0..3]) then
    popView.Items[ord(expl.ViewStyle)].Checked:=TRUE;
end;
//------------------------------------------------------------------------------------------
procedure TFileBrowsePanel.miViewStyleClick(Sender: TObject);
begin
  with TMenuItem(sender) do begin
    expl.ViewStyle:=TViewStyle(tag);
    Checked:=TRUE;
  end;
end;
//------------------------------------------------------------------------------------------
procedure TFileBrowsePanel.splitterCanResize(Sender: TObject;
  var NewSize: Integer; var Accept: Boolean);
begin
  Accept:=(NewSize>50);
end;
//------------------------------------------------------------------------------------------
procedure TFileBrowsePanel.btnCloseClick(Sender: TObject);
begin
  Visible:=FALSE;
end;
//------------------------------------------------------------------------------------------
procedure TFileBrowsePanel.explExecFile(Sender: TObject; Item: TListItem;
  var AllowExec: Boolean);
var
  IsDir :boolean;
begin
  if not Assigned(expl.Selected) then EXIT;

  IsDir:=Assigned(Item.Data) and (pFileRec(Item.Data)).IsDirectory;

  AllowExec:=Assigned(Item.Data) and IsDir;

  if Assigned(Item) and not IsDir then
    fmMain.OpenFile(expl.GetFullFileName(Item));
end;
//------------------------------------------------------------------------------------------
procedure TFileBrowsePanel.cbDiskChange(Sender: TObject);
begin
  expl.Path:=cbDisk.Drive+':\';
end;
//------------------------------------------------------------------------------------------
procedure TFileBrowsePanel.explLoaded(Sender: TObject);
begin
  labDir.DirName:=AddBackslash(expl.Path);
  labDir.Hint:=labDir.DirName;
end;
//------------------------------------------------------------------------------------------
procedure TFileBrowsePanel.lbFavOnKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  case Key of
    VK_RETURN:
      if (Shift=[]) then btnFavOpenClick(SELF);
    VK_DELETE:
      if (Shift=[]) then btnFavRemoveClick(SELF);
    VK_INSERT:
      if (Shift=[]) then btnFavAddClick(SELF);
  end;
end;
//------------------------------------------------------------------------------------------


////////////////////////////////////////////////////////////////////////////////////////////
//				   Custom draw
////////////////////////////////////////////////////////////////////////////////////////////
//------------------------------------------------------------------------------------------

end.

